#ifndef PGHP_SHAPE_H
#define PGHP_SHAPE_H

#include "ray.h"
#include "common.h"
#include "material.h"
#include "object.h"

/** represents a shape (geometry and material)
 */
class Shape : public Object
{
public:
    Shape() : m_transformation (Eigen::Affine3f::Identity().matrix()) {}

    Shape(const PropertyList&) : m_transformation(Eigen::Affine3f::Identity().matrix()) {}

    /** Search the nearest intersection between the ray and the shape.
      * It must be implemented in the derived class. */
    virtual bool intersect(const Ray& ray, Hit& hit) const {
        throw PGHPException("Shape::intersect must be implemented in the derived class"); }

    virtual const Material* material() const { return m_material; }
    virtual void setMaterial(const Material* mat) { m_material = mat; }

    virtual void setTransformation(const Eigen::Matrix4f& mat) { m_transformation = mat; }
    virtual const Transform& transformation() const { return m_transformation; }

    /// Register a child object (e.g. a material) with the shape
    virtual void addChild(Object *child);

    /// \brief Return the type of object provided by this instance
    EClassType getClassType() const { return EShape; }

    virtual std::string toString() const {
        throw PGHPException("Shape::toString must be implemented in the derived class"); }

protected:
    const Material* m_material = nullptr;
    Transform m_transformation;
};

#endif
